@@ -195,6 +195,7 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
     if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
     if (any(is.na(Dom))) stop("'Dom' has unknown values")
     if (is.null(names(Dom))) stop("'Dom' must be colnames")
+    Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
   }
   
   # Z
@@ -244,6 +245,23 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
   # Domains
   if (!is.null(Dom)) Y1 <- domain(Y, Dom) else Y1 <- Y
 
+  n_nonzero <- Y1[, lapply(.SD, function(x)  as.numeric(x!=0)), .SDcols = names(Y1)]
+  if (!is.null(period)){ n_nonzero <- data.table(period, n_nonzero) 
+                         n_nonzero <- n_nonzero[, lapply(.SD, sum), keyby=names(period), .SDcols=names(Y1)]
+                  } else n_nonzero <- n_nonzero[, lapply(.SD, sum), .SDcols=names(Y1)]
+
+  sample_size <- pop_size <- NULL
+  if (!is.null(Dom)) { if (!is.null(period)) {nhs <- data.table(Dom, period, sample_size=1, pop_size=w_final)
+                                              nhs <-  nhs[, lapply(.SD, sum, na.rm=T),
+                                                                    keyby=c(names(Dom), names(period)),
+                                                                   .SDcols=c("sample_size", "pop_size")]
+                                     } else { nhs <- data.table(Dom, sample_size=1, pop_size=w_final)
+                                              nhs <-  nhs[, lapply(.SD, sum, na.rm=T),
+                                                                    keyby=names(Dom),
+                                                                   .SDcols=c("sample_size", "pop_size")]
+                                  }
+                           } else nhs <- data.table(sample_size=nrow(Y1), pop_size=sum(w_final)) 
+
   # Design weights
   if (!is.null(X)) w_design <- w_final / g else w_design <- w_final
       
@@ -299,6 +317,9 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
   var_est <- transpos(var_est, is.null(period), "var_est", names(period))
   all_result <- var_est
 
+  n_nonzero <- transpos(n_nonzero, is.null(period), "n_nonzero", names(period))
+  all_result <- merge(all_result, n_nonzero, all=T)
+
   # Variance of HT estimator under current design
   var_cur_HT <- variance_othstr(Y=Y2a, H=H, H2=H2, 
                                 w_final=w_design, N_h=N_h,
@@ -306,7 +327,7 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
                                 period=period, dataset=NULL)
   var_cur_HT <- transpos(var_cur_HT, is.null(period), "var_cur_HT", names(period))
   all_result <- merge(all_result, var_cur_HT)
-  var_est <- var_cur_HT <- NULL
+  n_nonzero <- var_est <- var_cur_HT <- NULL
 
   # Variance of HT estimator under SRS
   if (is.null(period)) {
@@ -380,14 +401,14 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
 
   
   all_result[, estim:=Y_nov]   
-  all_result[!is.null(Z_nov), estim:=Y_nov/Z_nov]
+  if (!is.null(Z_nov)) all_result[, estim:=Y_nov/Z_nov]
 
   if (nrow(all_result[var_est < 0])>0) print("Estimation of variance are negative!")
  
-  # Effect of sample design
+  # Design effect of sample design
   all_result[, deff_sam:=var_cur_HT / var_srs_HT]
   
-  # Effect of estimator
+  # Design effect of estimator
   all_result[, deff_est:= var_est / var_cur_HT]
   
   # Overall effect of sample design and estimator
@@ -438,10 +459,17 @@ vardom_othstr <- function(Y, H, H2, PSU, w_final,
   
   if (!is.null(Z_nov)) { 
        all_result[, variable:=paste("R", get("variable"), sep="__", get("variableDZ"))] }
+  setkeyv(all_result, c(names(Dom), names(period)))
+
+  if (!is.null(Dom)) { all_result <- merge(all_result, nhs, all=T)
+                           } else { all_result[, sample_size:=nhs$sample_size]
+                                       all_result[, pop_size:=nhs$pop_size]} 
+
+  variab <- c("sample_size", "n_nonzero", "pop_size", "estim", "var", "se", 
+              "rse", "cv", "absolute_margin_of_error", "relative_margin_of_error",
+              "CI_lower", "CI_upper", "var_srs_HT",  "var_cur_HT", 
+              "var_srs_ca", "deff_sam", "deff_est", "deff")
 
-  variab <- c("estim", "var", "se", "rse", "cv", "absolute_margin_of_error",
-              "relative_margin_of_error", "CI_lower", "CI_upper", "var_srs_HT",
-              "var_cur_HT", "var_srs_ca", "deff_sam", "deff_est", "deff")
   setkeyv(all_result, c("nr_names", names(Dom), names(period)))
   all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=F]
 
